@@ -1,6 +1,6 @@
 ;;; gnosis-monkeytype.el --- Typing Module for Gnosis  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2023-2026  Free Software Foundation, Inc.
+;; Copyright (C) 2025  Thanos Apollo
 
 ;; Author: Thanos Apollo <public@thanosapollo.org>
 ;; Keywords: extensions
@@ -47,7 +47,7 @@
   "Face for correctly typed text."
   :group 'gnosis)
 
-(defcustom gnosis-monkeytype-themata '("basic" "cloze" "mc-cloze" "mcq")
+(defcustom gnosis-monkeytype-themata '("basic" "cloze" "mc-cloze")
   "Thema Types to monketype."
   :type '(repeat string)
   :group 'gnosis)
@@ -97,7 +97,6 @@
 (defun gnosis-monkeytype-exit ()
   "Exit monkeytyping."
   (interactive nil gnosis-monkeytype-mode)
-  (remove-hook 'after-change-functions #'gnosis-monkeytype--handler t)
   (kill-buffer (current-buffer))
   (ignore-errors (throw 'monkeytype-loop t))
   (exit-recursive-edit))
@@ -129,7 +128,7 @@ Optionally, highlight MISTAKES."
 	(fill-paragraph)
 	(switch-to-buffer (get-buffer-create gnosis-monkeytype-buffer-name))
 	(goto-char (point-min))
-	(add-hook 'after-change-functions #'gnosis-monkeytype--handler nil t)
+	(add-hook 'after-change-functions #'gnosis-monkeytype--handler)
 	(recursive-edit)
 	(setq gnosis-monkeytype-wpm-result
 	      (gnosis-monkeytype--calculate-wpm text-formatted start-time))))))
